@@ -103,7 +103,7 @@ sf_query_result_bulk <- function(job_id, batch_id, result_id,
                               httr_response$request$headers)
   }
   catch_errors(httr_response)
-  response_text <- content(httr_response, as="text", encoding="UTF-8")
+  response_text <- content(httr_response, type="text/plain", encoding="UTF-8")
   
   content_type <- httr_response$headers$`content-type`
   if (grepl('xml', content_type)) {
@@ -113,7 +113,8 @@ sf_query_result_bulk <- function(job_id, batch_id, result_id,
       res <- tibble()
     } else {
       cols_default <- if(guess_types) col_guess() else col_character()
-      res <- read_csv(response_text, col_types = cols(.default=cols_default))      
+      res<- content(httr_response, as="parsed", encoding="UTF-8", 
+                    col_types = cols(.default=cols_default))
     }
   } else {
     message(sprintf("Unhandled content-type: %s", content_type))
@@ -161,10 +162,8 @@ sf_query_bulk <- function(soql,
                           verbose = FALSE){
   
   if(is.null(object_name)){
-    object_name <- gsub("SELECT(.*) FROM ([A-Za-z_]+)\\b(.*)", "\\2", soql, ignore.case = TRUE)
-    message(sprintf("Guessed target object_name from query string: %s", object_name))
+    object_name <- guess_object_name_from_soql(soql)
   }
-  
   listed_objects <- sf_list_objects()
   valid_object_names <- sapply(listed_objects$sobjects, FUN=function(x){x$name})
   if(!object_name %in% valid_object_names){
