@@ -1,4 +1,4 @@
-## ---- echo = FALSE-------------------------------------------------------
+## ---- echo = FALSE------------------------------------------------------------
 NOT_CRAN <- identical(tolower(Sys.getenv("NOT_CRAN")), "true")
 knitr::opts_chunk$set(
   collapse = TRUE,
@@ -7,25 +7,34 @@ knitr::opts_chunk$set(
   eval = NOT_CRAN
 )
 
-## ----auth, include = FALSE-----------------------------------------------
+## ----auth, include = FALSE----------------------------------------------------
 suppressWarnings(suppressMessages(library(dplyr)))
 suppressWarnings(suppressMessages(library(here)))
 library(salesforcer)
 token_path <- here::here("tests", "testthat", "salesforcer_token.rds")
 suppressMessages(sf_auth(token = token_path, verbose = FALSE))
 
-## ----load-package, eval=FALSE--------------------------------------------
+## ----load-package, eval=FALSE-------------------------------------------------
 #  suppressWarnings(suppressMessages(library(dplyr)))
 #  library(salesforcer)
 #  sf_auth()
 
-## ----other-params, eval=FALSE--------------------------------------------
+## ----other-params1, eval=FALSE------------------------------------------------
 #  options(salesforcer.consumer_key = "012345678901-99thisisatest99connected33app22key")
 #  options(salesforcer.consumer_secret = "Th1s1sMyConsumerS3cr3t")
 #  
 #  sf_auth()
 
-## ------------------------------------------------------------------------
+## ----other-params2, eval=FALSE------------------------------------------------
+#  options(salesforcer.proxy_url = "64.251.21.73") # IP or a named domain
+#  options(salesforcer.proxy_port = 8080)
+#  options(salesforcer.proxy_username = "user")
+#  options(salesforcer.proxy_password = "pass")
+#  options(salesforcer.proxy_auth = "ntlm")
+#  
+#  sf_auth()
+
+## -----------------------------------------------------------------------------
 # pull down information of person logged in
 # it's a simple easy call to get started 
 # and confirm a connection to the APIs
@@ -33,20 +42,20 @@ user_info <- sf_user_info()
 sprintf("Organization Id: %s", user_info$organizationId)
 sprintf("User Id: %s", user_info$userId)
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 n <- 2
 new_contacts <- tibble(FirstName = rep("Test", n),
                        LastName = paste0("Contact-Create-", 1:n))
 created_records <- sf_create(new_contacts, "Contact")
 created_records
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 retrieved_records <- sf_retrieve(ids=created_records$id, 
                                  fields=c("FirstName", "LastName"), 
                                  object_name="Contact")
 retrieved_records
 
-## ----query-records-------------------------------------------------------
+## ----query-records------------------------------------------------------------
 my_soql <- sprintf("SELECT Id, 
                            Account.Name, 
                            FirstName, 
@@ -58,7 +67,7 @@ my_soql <- sprintf("SELECT Id,
 queried_records <- sf_query(my_soql)
 queried_records
 
-## ----update-records------------------------------------------------------
+## ----update-records-----------------------------------------------------------
 # Update some of those records
 queried_records <- queried_records %>%
   mutate(FirstName = "TestTest") %>% 
@@ -67,11 +76,11 @@ queried_records <- queried_records %>%
 updated_records <- sf_update(queried_records, object_name="Contact")
 updated_records
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 deleted_records <- sf_delete(updated_records$id)
 deleted_records
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 n <- 2
 new_contacts <- tibble(FirstName = rep("Test", n),
                        LastName = paste0("Contact-Create-", 1:n), 
@@ -91,6 +100,6 @@ upserted_records <- sf_upsert(input_data=upserted_contacts,
                               external_id_fieldname="My_External_Id__c")
 upserted_records
 
-## ---- include = FALSE----------------------------------------------------
-deleted_records <- sf_delete(upserted_records$id)
+## ---- include = FALSE---------------------------------------------------------
+deleted_records <- sf_delete(upserted_records$id, object_name = "Contact")
 
