@@ -11,8 +11,7 @@ test_that("testing SOSL", {
   searched_records1 <- sf_search(my_sosl, is_sosl=TRUE, api_type="SOAP")  
   searched_records2 <- sf_search(my_sosl, is_sosl=TRUE, api_type="REST") 
   expect_is(searched_records1, "tbl_df")
-  expect_named(searched_records1, c("sObject", "Id", "FirstName", 
-                                    "LastName", "My_External_Id__c"))
+  expect_named(searched_records1, c("sObject", "Id", "FirstName", "LastName", "My_External_Id__c"))
   expect_equal(names(searched_records1), names(searched_records2))
   expect_equal(nrow(searched_records1), nrow(searched_records2))
 })
